"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const react_dom_1 = require("react-dom");
const test_utils_1 = require("react-dom/test-utils");
const jest_image_snapshot_1 = require("jest-image-snapshot");
const configuration_1 = require("./configuration");
const change_tracker_1 = __importDefault(require("./change-tracker"));
const utils_1 = require("./utils");
expect.extend({ toMatchImageSnapshot: jest_image_snapshot_1.toMatchImageSnapshot });
const generateExamples = (options) => {
    const { sources } = configuration_1.get();
    if (!sources)
        throw new Error('Please configure an example source via `configure`.');
    return Object.values(sources).reduce((list, source) => {
        return [...list, ...source.execute(options)];
    }, []);
};
const extractCss = (container) => {
    return Array.from(container.querySelectorAll('style'))
        .map(el => {
        if (el.innerHTML)
            return el.innerHTML;
        if (!el.sheet)
            return '';
        const sheet = el.sheet;
        const rules = sheet.cssRules;
        return Array.from(rules)
            .map(r => r.cssText)
            .join('\n');
    })
        .join('\n');
};
const buildPage = (example) => {
    const container = document.body.appendChild(document.createElement('div'));
    const output = example.component();
    test_utils_1.act(() => {
        react_dom_1.render(output, container);
    });
    const body = document.body.outerHTML;
    const css = extractCss(document);
    react_dom_1.unmountComponentAtNode(container);
    if (container.parentNode === document.body) {
        document.body.removeChild(container);
    }
    return { body, css, name: example.name };
};
const getName = (targetName, exampleName) => `${targetName} ${exampleName}`.replace(/[^\w]+/gu, '-').toLowerCase();
const takeSnapshots = async ({ target, pages }) => {
    return await target.execute(pages);
};
function toMatchVisualSnapshot(received) {
    const context = this;
    const { testPath, snapshotState } = context;
    const { _updateSnapshot: updateSnapshot } = snapshotState;
    const examples = generateExamples(received);
    const { targets } = configuration_1.get();
    if (!targets)
        throw new Error('Please configure at least one browser target via `configure`.');
    const pages = examples.map(buildPage);
    const cachePath = utils_1.resolveCachePathFromTestPath(testPath);
    const changeTracker = change_tracker_1.default({ cachePath, updateSnapshot });
    for (const [targetName, target] of Object.entries(targets)) {
        const changedPages = changeTracker.getChangedPages(pages);
        const promises = takeSnapshots({ target, pages: changedPages });
        describe(`snapshot: ${targetName}`, () => {
            pages.forEach(page => {
                const customSnapshotIdentifier = getName(targetName, page.name);
                test(`${page.name}`, async () => {
                    const i = changedPages.indexOf(page);
                    if (i === -1)
                        return;
                    const images = await promises;
                    const image = images[i];
                    try {
                        expect(image).toMatchImageSnapshot({
                            customSnapshotIdentifier,
                        });
                    }
                    catch (error) {
                        changeTracker.markAsDirty(page);
                        throw error;
                    }
                });
            });
        });
        afterAll(() => {
            changeTracker.commitCache();
        });
    }
    return { pass: true, message: () => '' };
}
exports.toMatchVisualSnapshot = toMatchVisualSnapshot;
exports.runSnapshots = (options) => {
    let expectation = expect(options);
    if (!('toMatchVisualSnapshot' in expectation)) {
        expect.extend({ toMatchVisualSnapshot: toMatchVisualSnapshot });
        expectation = expect(options);
    }
    expectation.toMatchVisualSnapshot();
};
//# sourceMappingURL=jest.js.map